@@ -16,11 +16,12 @@
 #' level (`+` puts the right side *into* the left side). Using `-` might seem
 #' unintuitive if you think of the operator as "subtract", but look at it as a
 #' hyphen instead (the underlying reason is that `-` is the only operator in the
-#' same precedence group as `+`).
+#' same precedence group as `+`). An alternative and more explicit way to get
+#' the same effect as `-` is to use `merge()` on the left hand side.
 #'
 #' Often you are interested in creating single column or single row layouts.
 #' `patchwork` provides `|` (besides) and `/` (over) operators to support
-#' stacking and packing of plots. See the exampels for their use.
+#' stacking and packing of plots. See the examples for their use.
 #'
 #' In order to reduce code repetition `patchwork` provides two operators for
 #' adding ggplot elements (geoms, themes, facets, etc.) to multiple/all plots in
@@ -64,8 +65,10 @@ NULL
 #' @rdname plot_arithmetic
 #' @export
 "-.ggplot" <- function(e1, e2) {
-  if (is.grob(e2)) e2 <- wrap_elements(full = e2)
-  if (!is.ggplot(e2)) stop("Only knows how to fold ggplot objects together", call. = FALSE)
+  if (is.null(e2)) return(e1)
+  if (is.null(e1)) return(e2)
+  if (should_autowrap(e2)) e2 <- wrap_elements(full = e2)
+  if (!is.ggplot(e2)) cli_abort("Only knows how to fold ggplot objects together")
   patchwork <- new_patchwork()
   if (is_patchwork(e2)) {
     plot <- plot_filler()
@@ -80,7 +83,9 @@ NULL
 #' @rdname plot_arithmetic
 #' @export
 "/.ggplot" <- function(e1, e2) {
-  if (is.grob(e2)) e2 <- wrap_elements(full = e2)
+  if (is.null(e2)) return(e1)
+  if (is.null(e1)) return(e2)
+  if (should_autowrap(e2)) e2 <- wrap_elements(full = e2)
   if (!is_patchwork(e1)) {
     e1 + e2 + plot_layout(ncol = 1)
   } else if (!is.null(e1$patches$layout$ncol) && e1$patches$layout$ncol == 1) {
@@ -93,7 +98,9 @@ NULL
 #' @rdname plot_arithmetic
 #' @export
 "|.ggplot" <- function(e1, e2) {
-  if (is.grob(e2)) e2 <- wrap_elements(full = e2)
+  if (is.null(e2)) return(e1)
+  if (is.null(e1)) return(e2)
+  if (should_autowrap(e2)) e2 <- wrap_elements(full = e2)
   if (!is_patchwork(e1)) {
     e1 + e2 + plot_layout(nrow = 1)
   } else if (!is.null(e1$patches$layout$nrow) && e1$patches$layout$nrow == 1) {
@@ -105,6 +112,7 @@ NULL
 #' @rdname plot_arithmetic
 #' @export
 "*.gg" <- function(e1, e2) {
+  if (is.null(e2)) return(e1)
   if (is_patchwork(e1)) {
     e1$patches$plots <- lapply(e1$patches$plots, function(p) {
       if (!is_patchwork(p)) p <- p + e2
@@ -117,6 +125,7 @@ NULL
 #' @importFrom ggplot2 is.theme
 #' @export
 "&.gg" <- function(e1, e2) {
+  if (is.null(e2)) return(e1)
   if (is_patchwork(e1)) {
     if (is.theme(e2)) {
       e1$patches$annotation$theme <- e1$patches$annotation$theme + e2
@@ -132,3 +141,9 @@ NULL
   }
   e1 + e2
 }
+
+#' @export
+"*.ggplot2::gg" <- `*.gg`
+
+#' @export
+"&.ggplot2::gg" <- `&.gg`
